@@ -34,11 +34,14 @@ import platform
 import traceback
 import json
 
-from PyQt5.QtCore import PYQT_VERSION_STR
-from PyQt5.QtCore import QT_VERSION_STR
-from PyQt5.QtCore import pyqtSlot
+from PyQt5.QtCore import PYQT_VERSION_STR, QT_VERSION_STR, pyqtSlot
 from PyQt5.QtWidgets import QApplication, QStyleFactory, QMessageBox
 
+# Disable sandbox support for QtWebEngine (required on some Linux distros
+# for the QtWebEngineWidgets to be rendered, otherwise no timeline is visible).
+# https://doc.qt.io/qt-5/qtwebengine-platform-notes.html#sandboxing-support
+os.environ["QTWEBENGINE_DISABLE_SANDBOX"] = "1"
+
 
 def get_app():
     """ Get the current QApplication instance of OpenShot """
@@ -72,35 +75,33 @@ class StartupError:
 
 
 class OpenShotApp(QApplication):
-    """ This class is the primary QApplication for OpenShot.
-            mode=None (normal), mode=unittest (testing)"""
+    """The primary QApplication subclass for OpenShot."""
 
-    def __init__(self, *args, mode=None):
-        QApplication.__init__(self, *args)
-        self.mode = mode or "normal"
+    def __init__(self, *args, **kwargs):
+        self.mode = kwargs.pop("mode", None)
+        super().__init__(*args, **kwargs)
         self.args = super().arguments()
         self.errors = []
 
         try:
             # Import modules
-            from classes import info, sentry
+            from classes import info
             from classes.logger import log, reroute_output
 
             # Log the session's start
-            if mode != "unittest":
+            if self.mode != "unittest":
                 import time
                 log.info("-" * 48)
                 log.info(time.asctime().center(48))
                 log.info('Starting new session'.center(48))
 
-            log.debug("Starting up in {} mode".format(self.mode))
-            log.debug("Command line: {}".format(self.args))
+            log.debug("Command line: %s", self.args)
 
-            from classes import settings, project_data, updates
+            from classes import settings, project_data, updates, sentry
             import openshot
 
             # Re-route stdout and stderr to logger
-            if mode != "unittest":
+            if self.mode != "unittest":
                 reroute_output()
 
         except ImportError as ex:
@@ -136,8 +137,15 @@ class OpenShotApp(QApplication):
         # Set location of OpenShot program (for libopenshot)
         openshot.Settings.Instance().PATH_OPENSHOT_INSTALL = info.PATH
 
+        # Set BABL extensions path
+        babl_ext_path = os.path.join(info.PATH, "lib", "babl-ext")
+        log.info(f"checking babl_ext_path: {babl_ext_path}")
+        if os.path.exists(babl_ext_path):
+            os.environ["BABL_PATH"] = babl_ext_path
+            log.info(f"setting BABL_PATH: {babl_ext_path}")
+
         # Check to disable sentry
-        if not self.settings.get('send_metrics'):
+        if self.mode == "unittest" or not self.settings.get('send_metrics'):
             sentry.disable_tracing()
 
     def show_environment(self, info, openshot):
@@ -187,6 +195,10 @@ class OpenShotApp(QApplication):
         ))
 
     def gui(self):
+        """
+        Initialize GUI and main window.
+        :return: bool: True if the GUI has no errors, False if we fail to initialize the GUI
+        """
         from classes import language, sentry, ui_util, logger_libopenshot
         from PyQt5.QtGui import QFont, QFontDatabase as QFD
 
@@ -262,7 +274,12 @@ class OpenShotApp(QApplication):
         # Create main window
         from windows.main_window import MainWindow
         log.debug("Creating main interface window")
-        self.window = MainWindow(mode=self.mode)
+        self.window = MainWindow()
+
+        # Check for gui launch failures
+        if self.mode == "quit":
+            self.window.close()
+            return False
 
         # Clear undo/redo history
         self.window.updateStatusChanged(False, False)
@@ -274,19 +291,20 @@ class OpenShotApp(QApplication):
         if len(args) < 2:
             # Recover backup file (this can't happen until after the Main Window has completely loaded)
             self.window.RecoverBackup.emit()
-            return
+            return True
 
-        log.info('Process command-line arguments: %s' % args)
+        log.info('Process command-line arguments: %s', args[1:])
 
         # Auto load project if passed as argument
         if args[1].endswith(".osp"):
             self.window.OpenProjectSignal.emit(args[1])
-            return
+            return True
 
         # Start a new project and auto import any media files
         self.project.load("")
         for arg in args[1:]:
             self.window.filesView.add_file(arg)
+        return True
 
     def settings_load_error(self, filepath=None):
         """Use QMessageBox to warn the user of a settings load issue"""
@@ -338,3 +356,4 @@ def onLogTheEnd():
         import logging
         log = logging.getLogger(".")
         log.debug('Failed to write session ended log', exc_info=1)
+
