@@ -40,7 +40,7 @@ App.controller("TimelineCtrl", function ($scope) {
     duration: 300, //length of project in seconds
     scale: 16.0, //seconds per tick
     tick_pixels: 100, //pixels between tick mark
-    playhead_position: 10, //position of play head
+    playhead_position: 0.0, //position of play head
     clips: [],
     effects: [],
     layers: [
@@ -64,6 +64,7 @@ App.controller("TimelineCtrl", function ($scope) {
   $scope.enable_snapping = true;
   $scope.enable_razor = false;
   $scope.enable_playhead_follow = true;
+  $scope.keyframe_prop_filter = "";
   $scope.debug = false;
   $scope.min_width = 1024;
   $scope.track_label = "Track %s";
@@ -138,6 +139,28 @@ App.controller("TimelineCtrl", function ($scope) {
     }
   };
 
+  // Get keyframe interpolation type
+  $scope.lookupInterpolation = function(interpolation_number) {
+    if (parseInt(interpolation_number) === 0) {
+      return "bezier";
+    } else if (parseInt(interpolation_number) === 1) {
+      return "linear";
+    } else {
+      return "constant";
+    }
+  }
+
+  // Seek to keyframe
+  $scope.selectPoint = function(object, point) {
+    var frames_per_second = $scope.project.fps.num / $scope.project.fps.den;
+    var clip_position_frames = object.position * frames_per_second;
+    var absolute_seek_frames = clip_position_frames + parseInt(point) - (object.start * frames_per_second);
+
+    if ($scope.Qt) {
+      timeline.SeekToKeyframe(absolute_seek_frames)
+    }
+  }
+
   // Get an array of keyframe points for the selected clips
   $scope.getKeyframes = function (object) {
     // List of keyframes
@@ -150,26 +173,33 @@ App.controller("TimelineCtrl", function ($scope) {
     // Loop through properties of an object (clip/transition), looking for keyframe points
     for (var child in object) {
       if (!object.hasOwnProperty(child)) {
-        //The current property is not a direct property of p
+        //The current property is not a direct property
+        continue;
+      }
+      if ($scope.keyframe_prop_filter.length > 0 &&
+        !child.toLowerCase().includes($scope.keyframe_prop_filter.toLowerCase())) {
+        // Not the current filter property
         continue;
       }
       // Determine if this property is a Keyframe
-      if (typeof object[child] === "object" && "Points" in object[child]) {
+      if (typeof object[child] === "object" && "Points" in object[child] && object[child].Points.length > 1) {
         for (var point = 0; point < object[child].Points.length; point++) {
           var co = object[child].Points[point].co;
+          var interpolation = $scope.lookupInterpolation(object[child].Points[point].interpolation);
           if (co.X >= clip_start_x && co.X <= clip_end_x) {
             // Only add keyframe coordinates that are within the bounds of the clip
-            keyframes[co.X] = co.Y;
+            keyframes[co.X] = interpolation;
           }
         }
       }
       // Determine if this property is a Color Keyframe
-      if (typeof object[child] === "object" && "red" in object[child]) {
+      if (typeof object[child] === "object" && "red" in object[child] && object[child]["red"].Points.length > 1) {
         for (var color_point = 0; color_point < object[child]["red"].Points.length; color_point++) {
           var color_co = object[child]["red"].Points[color_point].co;
+          var color_interpolation = $scope.lookupInterpolation(object[child]["red"].Points[color_point].interpolation);
           if (color_co.X >= clip_start_x && color_co.X <= clip_end_x) {
             // Only add keyframe coordinates that are within the bounds of the clip
-            keyframes[color_co.X] = color_co.Y;
+            keyframes[color_co.X] = color_interpolation;
           }
         }
       }
@@ -180,32 +210,43 @@ App.controller("TimelineCtrl", function ($scope) {
         // Loop through properties of an effect, looking for keyframe points
         for (var effect_prop in object["effects"][effect]) {
           if (!object["effects"][effect].hasOwnProperty(effect_prop)) {
-            //The current property is not a direct property of p
+            //The current property is not a direct property
+            continue;
+          }
+          if ($scope.keyframe_prop_filter.length > 0 &&
+            !effect_prop.toLowerCase().includes($scope.keyframe_prop_filter.toLowerCase())) {
+            // Not the current filter property
             continue;
           }
           // Determine if this property is a Keyframe
-          if (typeof object["effects"][effect][effect_prop] === "object" && "Points" in object["effects"][effect][effect_prop]) {
+          if (typeof object["effects"][effect][effect_prop] === "object"
+            && "Points" in object["effects"][effect][effect_prop] && object["effects"][effect][effect_prop].Points.length > 1) {
             for (var effect_point = 0; effect_point < object["effects"][effect][effect_prop].Points.length; effect_point++) {
               var effect_co = object["effects"][effect][effect_prop].Points[effect_point].co;
+              var effect_interpolation = $scope.lookupInterpolation(object["effects"][effect][effect_prop].Points[effect_point].interpolation);
               if (effect_co.X >= clip_start_x && effect_co.X <= clip_end_x) {
                 // Only add keyframe coordinates that are within the bounds of the clip
-                keyframes[effect_co.X] = effect_co.Y;
+                keyframes[effect_co.X] = effect_interpolation;
               }
             }
           }
           // Determine if this property is a Color Keyframe
-          if (typeof object["effects"][effect][effect_prop] === "object" && "red" in object["effects"][effect][effect_prop]) {
+          if (typeof object["effects"][effect][effect_prop] === "object"
+            && "red" in object["effects"][effect][effect_prop]
+            && object["effects"][effect][effect_prop]["red"].Points.length > 1) {
             for (var effect_color_point = 0; effect_color_point < object["effects"][effect][effect_prop]["red"].Points.length; effect_color_point++) {
               var effect_color_co = object["effects"][effect][effect_prop]["red"].Points[effect_color_point].co;
+              var effect_color_interpolation = $scope.lookupInterpolation(object["effects"][effect][effect_prop]["red"].Points[effect_color_point].interpolation);
               if (effect_color_co.X >= clip_start_x && effect_color_co.X <= clip_end_x) {
                 // Only add keyframe coordinates that are within the bounds of the clip
-                keyframes[effect_color_co.X] = effect_color_co.Y;
+                keyframes[effect_color_co.X] = effect_color_interpolation;
               }
             }
           }
         }
       }
     }
+
     // Return keyframe array
     return keyframes;
   };
@@ -331,60 +372,22 @@ App.controller("TimelineCtrl", function ($scope) {
     clip_selector.attr("src", existing_thumb_path);
   };
 
-  // Set the audio data for a clip
-  $scope.setAudioData = function (clip_id, audio_data) {
-    // Find matching clip
-    for (var clip_index = 0; clip_index < $scope.project.clips.length; clip_index++) {
-      if ($scope.project.clips[clip_index].id === clip_id) {
-        // Set audio data
-        $scope.$apply(function () {
-          $scope.project.clips[clip_index].audio_data = audio_data;
-          $scope.project.clips[clip_index].show_audio = true;
-        });
-        timeline.qt_log("DEBUG", "Audio data successful set on clip JSON");
-        break;
-      }
-
-      // Draw audio data
-      drawAudio($scope, clip_id);
-    }
-  };
-
-  // Hide the audio waveform for a clip
-  $scope.hideAudioData = function (clip_id) {
-    // Find matching clip
-    for (var clip_index = 0; clip_index < $scope.project.clips.length; clip_index++) {
-      if ($scope.project.clips[clip_index].id === clip_id) {
-        // Set audio data
-        $scope.$apply(function () {
-          $scope.project.clips[clip_index].show_audio = false;
-          $scope.project.clips[clip_index].audio_data = [];
-        });
-        break;
-      }
-    }
-  };
-
-  // Redraw all audio waveforms on the timeline (if any)
+  // Redraw all audio waveforms on the timeline (for example, if the screen is resized)
   $scope.reDrawAllAudioData = function () {
-    // Find matching clip
+    // Loop through all clips (and look for audio data)
     for (var clip_index = 0; clip_index < $scope.project.clips.length; clip_index++) {
-      if ("audio_data" in $scope.project.clips[clip_index] && $scope.project.clips[clip_index].audio_data.length > 0) {
-        // Redraw audio data (since it has audio data)
+      if ("ui" in $scope.project.clips[clip_index] && "audio_data" in $scope.project.clips[clip_index].ui
+        && $scope.project.clips[clip_index].ui.audio_data.length > 1) {
+        // Redraw audio data
         drawAudio($scope, $scope.project.clips[clip_index].id);
       }
     }
   };
 
-  // Does clip have audio_data?
-  $scope.hasAudioData = function (clip_id) {
-    // Find matching clip
-    for (var clip_index = 0; clip_index < $scope.project.clips.length; clip_index++) {
-      if ($scope.project.clips[clip_index].id === clip_id && "audio_data" in $scope.project.clips[clip_index] && $scope.project.clips[clip_index].audio_data.length > 0) {
-        return true;
-      }
-    }
-    return false;
+  $scope.setPropertyFilter = function (property) {
+    $scope.$apply(function () {
+      $scope.keyframe_prop_filter = property;
+    });
   };
 
   // Change the snapping mode
@@ -417,20 +420,60 @@ App.controller("TimelineCtrl", function ($scope) {
   // Get the color of an effect
   $scope.getEffectColor = function (effect_type) {
     switch (effect_type) {
+      case "Bars":
+        return "#4d7bff";
       case "Blur":
         return "#0095bf";
       case "Brightness":
         return "#5500ff";
+      case "Caption":
+        return "#5e7911";
       case "ChromaKey":
         return "#00ad2d";
+      case "Color Shift":
+        return "#b39373";
+      case "Compressor":
+        return "#A52A2A";
+      case "Crop":
+        return "#7b3f00";
       case "Deinterlace":
         return "#006001";
+      case "Delay":
+        return "#ff4dd4";
+      case "Distortion":
+        return "#7393B3";
+      case "Echo":
+        return "#5C4033";
+      case "Expander":
+        return "#C4A484";
+      case "Hue":
+        return "#2d7b6b";
       case "Mask":
         return "#cb0091";
       case "Negate":
         return "#ff9700";
+      case "Noise":
+        return "#a9a9a9";
+      case "ObjectDetection":
+        return "#636363";
+      case "Parametric EQ":
+        return "#708090";
+      case "Pixelate":
+        return	"#9fa131";
+      case "Robotization":
+        return "#CC5500";
       case "Saturation":
         return "#ff3d00";
+      case "Shift":
+        return "#8d7960";
+      case "Stabilizer":
+        return "#9F2B68";
+      case "Tracker":
+        return "#DE3163";
+      case "Wave":
+        return "#FF00Ff";
+      case "Whisperization":
+        return "#93914a";
       default:
         return "#000000";
     }
@@ -457,7 +500,6 @@ App.controller("TimelineCtrl", function ($scope) {
 
   // Update cache json
   $scope.renderCache = function (cache_json) {
-    // Push new clip onto stack
     $scope.project.progress = cache_json;
 
     //clear the canvas first
@@ -467,26 +509,27 @@ App.controller("TimelineCtrl", function ($scope) {
 
     // Determine fps & and get cached ranges
     var fps = $scope.project.fps.num / $scope.project.fps.den;
-    var progress = $scope.project.progress.ranges;
+    if ($scope.project.progress && $scope.project.progress.hasOwnProperty('ranges')) {
+        // Loop through each cached range of frames, and draw rect
+        let progress = $scope.project.progress.ranges;
+        for (var p = 0; p < progress.length; p++) {
+          //get the progress item details
+          var start_second = parseFloat(progress[p]["start"]) / fps;
+          var stop_second = parseFloat(progress[p]["end"]) / fps;
 
-    // Loop through each cached range of frames, and draw rect
-    for (var p = 0; p < progress.length; p++) {
-      //get the progress item details
-      var start_second = parseFloat(progress[p]["start"]) / fps;
-      var stop_second = parseFloat(progress[p]["end"]) / fps;
-
-      //figure out the actual pixel position, constrained by max width
-      var start_pixel = $scope.canvasMaxWidth(start_second * $scope.pixelsPerSecond);
-      var stop_pixel = $scope.canvasMaxWidth(stop_second * $scope.pixelsPerSecond);
-      var rect_length = stop_pixel - start_pixel;
-      if (rect_length < 1) {
-        break;
-      }
-      //get the element and draw the rects
-      ctx.beginPath();
-      ctx.rect(start_pixel, 0, rect_length, 5);
-      ctx.fillStyle = "#4B92AD";
-      ctx.fill();
+          //figure out the actual pixel position, constrained by max width
+          var start_pixel = $scope.canvasMaxWidth(start_second * $scope.pixelsPerSecond);
+          var stop_pixel = $scope.canvasMaxWidth(stop_second * $scope.pixelsPerSecond);
+          var rect_length = stop_pixel - start_pixel;
+          if (rect_length < 1) {
+            continue;
+          }
+          //get the element and draw the rects
+          ctx.beginPath();
+          ctx.rect(start_pixel, 0, rect_length, 5);
+          ctx.fillStyle = "#4B92AD";
+          ctx.fill();
+        }
     }
   };
 
@@ -536,11 +579,10 @@ App.controller("TimelineCtrl", function ($scope) {
       $scope.selectTransition("", true);
     }
     // Call slice method and exit (don't actually select the clip)
-    if (id !== "" && $scope.enable_razor) {
-      if ($scope.Qt) {
-        var cursor_seconds = $scope.getJavaScriptPosition(event.clientX, null).position;
-        timeline.RazorSliceAtCursor(id, "", cursor_seconds);
-      }
+    if (id !== "" && $scope.enable_razor && $scope.Qt && typeof event !== 'undefined') {
+      var cursor_seconds = $scope.getJavaScriptPosition(event.clientX, null).position;
+      timeline.RazorSliceAtCursor(id, "", cursor_seconds);
+
       // Don't actually select clip
       return;
     }
@@ -590,11 +632,10 @@ App.controller("TimelineCtrl", function ($scope) {
       $scope.selectClip("", true);
     }
     // Call slice method and exit (don't actually select the transition)
-    if (id !== "" && $scope.enable_razor) {
-      if ($scope.Qt) {
-        var cursor_seconds = $scope.getJavaScriptPosition(event.clientX, null).position;
-        timeline.RazorSliceAtCursor("", id, cursor_seconds);
-      }
+    if (id !== "" && $scope.enable_razor && $scope.Qt && typeof event !== 'undefined') {
+      var cursor_seconds = $scope.getJavaScriptPosition(event.clientX, null).position;
+      timeline.RazorSliceAtCursor("", id, cursor_seconds);
+
       // Don't actually select transition
       return;
     }
@@ -633,7 +674,6 @@ App.controller("TimelineCtrl", function ($scope) {
 
   // Format the thumbnail path: http://127.0.0.1:8081/thumbnails/FILE-ID/FRAME-NUMBER/
   /**
-   * @return {string}
    * @return {string}
    */
   $scope.getThumbPath = function (clip) {
@@ -658,9 +698,12 @@ App.controller("TimelineCtrl", function ($scope) {
     return Math.min(32767, desired_width);
   };
 
-// Find the furthest right edge on the timeline (and resize it if too small)
+// Find the furthest right edge on the timeline (and resize it if too small or too large)
   $scope.resizeTimeline = function () {
-    // Unselect all clips
+    let max_timeline_padding = 20;
+    let min_timeline_padding = 10;
+    let min_timeline_length = 300; // Length of the default OpenShot project
+    // Find latest end of a clip
     var furthest_right_edge = 0;
     for (var clip_index = 0; clip_index < $scope.project.clips.length; clip_index++) {
       var clip = $scope.project.clips[clip_index];
@@ -670,10 +713,10 @@ App.controller("TimelineCtrl", function ($scope) {
       }
     }
     // Resize timeline
-    if (furthest_right_edge > $scope.project.duration) {
+    if (furthest_right_edge > $scope.project.duration - min_timeline_padding || furthest_right_edge < $scope.project.duration - max_timeline_padding) {
       if ($scope.Qt) {
-        timeline.resizeTimeline(furthest_right_edge + 10);
-        $scope.project.duration = furthest_right_edge + 10;
+        let new_timeline_length = Math.max(min_timeline_length, furthest_right_edge + min_timeline_padding);
+        timeline.resizeTimeline(new_timeline_length);
       }
     }
   };
@@ -681,57 +724,71 @@ App.controller("TimelineCtrl", function ($scope) {
 // Show clip context menu
   $scope.showClipMenu = function (clip_id, event) {
     if ($scope.Qt && !$scope.enable_razor) {
-      timeline.qt_log("DEBUG", "$scope.showClipMenu");
-      $scope.selectClip(clip_id, false, event);
-      timeline.ShowClipMenu(clip_id);
+      setTimeout(function() {
+        timeline.qt_log("DEBUG", "$scope.showClipMenu");
+        $scope.selectClip(clip_id, false, event);
+        timeline.ShowClipMenu(clip_id);
+      });
     }
   };
 
 // Show clip context menu
   $scope.showEffectMenu = function (effect_id) {
     if ($scope.Qt && !$scope.enable_razor) {
-      timeline.qt_log("DEBUG", "$scope.showEffectMenu");
-      timeline.ShowEffectMenu(effect_id);
+      setTimeout(function() {
+        timeline.qt_log("DEBUG", "$scope.showEffectMenu");
+        timeline.ShowEffectMenu(effect_id);
+      });
     }
   };
 
 // Show transition context menu
   $scope.showTransitionMenu = function (tran_id, event) {
     if ($scope.Qt && !$scope.enable_razor) {
-      timeline.qt_log("DEBUG", "$scope.showTransitionMenu");
-      $scope.selectTransition(tran_id, false, event);
-      timeline.ShowTransitionMenu(tran_id);
+      setTimeout(function() {
+        timeline.qt_log("DEBUG", "$scope.showTransitionMenu");
+        $scope.selectTransition(tran_id, false, event);
+        timeline.ShowTransitionMenu(tran_id);
+      });
     }
   };
 
 // Show track context menu
   $scope.showTrackMenu = function (layer_id) {
     if ($scope.Qt && !$scope.enable_razor) {
-      timeline.qt_log("DEBUG", "$scope.showTrackMenu");
-      timeline.ShowTrackMenu(layer_id);
+      setTimeout(function() {
+        timeline.qt_log("DEBUG", "$scope.showTrackMenu");
+        timeline.ShowTrackMenu(layer_id);
+      });
     }
   };
 
 // Show marker context menu
   $scope.showMarkerMenu = function (marker_id) {
     if ($scope.Qt && !$scope.enable_razor) {
-      timeline.qt_log("DEBUG", "$scope.showMarkerMenu");
-      timeline.ShowMarkerMenu(marker_id);
+      setTimeout(function() {
+        timeline.qt_log("DEBUG", "$scope.showMarkerMenu");
+        timeline.ShowMarkerMenu(marker_id);
+      });
     }
   };
 
   // Show playhead context menu
   $scope.showPlayheadMenu = function (position) {
     if ($scope.Qt && !$scope.enable_razor) {
-      timeline.qt_log("DEBUG", "$scope.showPlayheadMenu");
-      timeline.ShowPlayheadMenu(position);
+      setTimeout(function() {
+        timeline.qt_log("DEBUG", "$scope.showPlayheadMenu");
+        timeline.ShowPlayheadMenu(position);
+      });
     }
   };
 
   // Show timeline context menu
   $scope.showTimelineMenu = function (e, layer_number) {
     if ($scope.Qt && !$scope.enable_razor) {
-      timeline.ShowTimelineMenu($scope.getJavaScriptPosition(e.pageX, null).position, layer_number);
+      setTimeout(function() {
+        timeline.ShowTimelineMenu($scope.getJavaScriptPosition(e.pageX, null).position, layer_number);
+      });
     }
   };
 
@@ -802,7 +859,7 @@ App.controller("TimelineCtrl", function ($scope) {
   };
 
   // Get JSON of most recent item (used by Qt)
-  $scope.updateRecentItemJSON = function (item_type, item_id) {
+  $scope.updateRecentItemJSON = function (item_type, item_id, item_tid) {
 
     // Find item in JSON
     var item_object = null;
@@ -830,10 +887,10 @@ App.controller("TimelineCtrl", function ($scope) {
 
     // update clip in Qt (very important =)
     if (item_type === "clip") {
-      timeline.update_clip_data(JSON.stringify(item_object), true, true, false);
+      timeline.update_clip_data(JSON.stringify(item_object), true, true, false, item_tid);
     }
     else if (item_type === "transition") {
-      timeline.update_transition_data(JSON.stringify(item_object), true, false);
+      timeline.update_transition_data(JSON.stringify(item_object), true, false, item_tid);
     }
 
     // Resize timeline if it's too small to contain all clips
@@ -1224,11 +1281,15 @@ App.controller("TimelineCtrl", function ($scope) {
    */
   $scope.getTrackAtY = function (y) {
     // Loop through each layer (looking for the closest track based on Y coordinate)
+    let tolerance = 24;
     for (var layer_index = $scope.project.layers.length - 1; layer_index >= 0; layer_index--) {
       var layer = $scope.project.layers[layer_index];
 
       // Compare position of track to Y param and return first matching layer
-      if (layer.y > y) {
+      // Consider a tolerance around the cursor position
+      if (layer.y > (y - tolerance)) {
+        return layer;
+      } else if (layer.y > (y + tolerance)) {
         return layer;
       }
     }
@@ -1279,6 +1340,34 @@ App.controller("TimelineCtrl", function ($scope) {
     return style;
   };
 
+  // Determine which z-index to assign for a clip / transition.
+  // Selected items z-index should be larger than unselected items.
+  // The index is passed in, and represents the current position
+  // in the render loop (1, 2, 3, etc...)
+  $scope.getZindex = function (item, starting_index, index) {
+    let unselected_zindex = starting_index;
+    let selected_zindex = starting_index + 1000;
+
+    if (item.selected) {
+      return selected_zindex + index;
+    } else {
+      return unselected_zindex + index;
+    }
+  };
+
+  $scope.markerPath = function(marker) {
+    var dir = "media/images/markers";
+    var marker_file;
+    if(marker.hasOwnProperty("vector")) {
+      marker_file = `${marker.vector}.svg`;
+    } else if(marker.icon === "blue.png") {
+      marker_file = "blue.svg";
+    } else {
+      marker_file = marker.icon;
+    }
+    return `${dir}/${marker_file}`;
+  };
+
   // Apply JSON diff from UpdateManager (this is how the user interface communicates changes
   // to the timeline. A change can be an insert, update, or delete. The change is passed in
   // as JSON, which represents the change.
@@ -1365,10 +1454,7 @@ App.controller("TimelineCtrl", function ($scope) {
           // Update: If action and current object are Objects
           if (current_object.constructor === Object && action.value.constructor === Object) {
             for (var update_key in action.value) {
-              if (update_key in current_object) {
-                // Only copy over keys that exist in both action and current_object
-                current_object[update_key] = action.value[update_key];
-              }
+              current_object[update_key] = action.value[update_key];
             }
           }
           else {
@@ -1398,6 +1484,10 @@ App.controller("TimelineCtrl", function ($scope) {
         $scope.updateLayerIndex();
       }
     }
+
+    // Apply all changes
+    $scope.$apply();
+
     // return true
     return true;
   };
@@ -1430,8 +1520,11 @@ App.controller("TimelineCtrl", function ($scope) {
       scrollLeft: 0
     }, "slow");
 
-    // Update playhead position and time readout
-    $scope.movePlayhead($scope.project.playhead_position);
+    // Update playhead position and time readout (reset to zero)
+    $scope.movePlayhead(0.0);
+
+    // Apply all changes
+    $scope.$apply();
 
     // return true
     return true;
@@ -1466,8 +1559,9 @@ App.controller("TimelineCtrl", function ($scope) {
         show_audio: false,
         alpha: {Points: []},
         location_x: {Points: [
-				  {co: {X: 1.0, Y: -0.5}},
-          {co: {X: 30.0, Y: -0.4}}
+				  {co: {X: 1.0, Y: -0.5}, interpolation: 0},
+          {co: {X: 30.0, Y: -0.4}, interpolation: 1},
+          {co: {X: 100.0, Y: -0.4}, interpolation: 2}
         ]},
         location_y: {Points: []},
         scale_x: {Points: []},
@@ -1498,7 +1592,8 @@ App.controller("TimelineCtrl", function ($scope) {
   $scope.addMarker = function (markLoc) {
     $scope.project.markers.push({
       position: parseInt(markLoc, 10),
-      icon: "blue.png"
+      icon: "blue.png",
+      vector: "blue"
     });
     $scope.markLoc = "";
   };
