@@ -27,18 +27,13 @@
  """
 
 import os
-from functools import partial
 
-from PyQt5.QtCore import *
-from PyQt5.QtWidgets import *
+from PyQt5.QtWidgets import QDialog
 
 from classes import info, ui_util
-from classes.logger import log
 from classes.app import get_app
-from classes.metrics import *
-from windows.views.credits_treeview import CreditsTreeView
+from classes.metrics import track_metric_screen
 
-import json
 
 class Animation(QDialog):
     """ Animation Dialog """
