@@ -27,27 +27,52 @@
 
 import time
 import sip
+import math
 
-from PyQt5.QtCore import QObject, QThread, pyqtSlot, pyqtSignal, QCoreApplication
+from PyQt5.QtCore import QObject, QThread, QTimer, pyqtSlot, pyqtSignal, QCoreApplication
 from PyQt5.QtWidgets import QMessageBox
 import openshot  # Python module for libopenshot (required video editing module installed separately)
 
 from classes.app import get_app
 from classes.logger import log
+from classes.updates import UpdateInterface
 
 
-class PreviewParent(QObject):
+class PreviewParent(QObject, UpdateInterface):
     """ Class which communicates with the PlayerWorker Class (running on a separate thread) """
 
+    def changed(self, action):
+        """ This method is invoked by the UpdateManager each time a change happens (i.e UpdateInterface) """
+
+        # Ignore changes that don't affect libopenshot
+        if len(action.key) >= 1 and action.key[0].lower() in ["files", "history", "markers", "layers",
+                                                              "scale", "profile", "sample_rate"]:
+            return
+
+        try:
+            # Keep track of max timeline frame # on any updates to the timeline
+            self.timeline_max_length = self.timeline.GetMaxFrame()
+            log.debug(f"Max timeline length/frames detected: {self.timeline_max_length}")
+
+        except Exception as e:
+            log.info("Error calculating max timeline length on PreviewParent: %s. %s" % (e, action.json(is_array=True)))
+
     # Signal when the frame position changes in the preview player
     def onPositionChanged(self, current_frame):
         self.parent.movePlayhead(current_frame)
 
         # Check if we are at the end of the timeline
-        if self.worker.player.Mode() == openshot.PLAYBACK_PLAY and current_frame >= self.worker.timeline_length and self.worker.timeline_length != -1:
-            # Yes, pause the video
-            self.parent.actionPlay.trigger()
-            self.worker.timeline_length = -1
+        if self.worker.player.Mode() == openshot.PLAYBACK_PLAY:
+            if self.worker.player.Speed() > 0.0 and current_frame >= self.timeline_max_length:
+                # Yes, pause the video
+                self.parent.PauseSignal.emit()
+                # If the player got past the end of the project, go back.
+                self.worker.Seek(self.timeline_max_length)
+            if self.worker.player.Speed() < 0.0 and current_frame <= 1:
+                # If rewinding, and the player got past the first frame,
+                # pause and go to frame 1
+                self.parent.PauseSignal.emit()
+                self.worker.Seek(1)
 
     # Signal when the playback mode changes in the preview player (i.e PLAY, PAUSE, STOP)
     def onModeChanged(self, current_mode):
@@ -63,20 +88,28 @@ class PreviewParent(QObject):
 
     # Signal when the playback encounters an error
     def onError(self, error):
-        log.warning('Player error: %s', error)
-
         # Get translation object
         _ = get_app()._tr
 
         # Only JUCE audio errors bubble up here now
-        if get_app().window.mode != "unittest":
-            QMessageBox.warning(self.parent, _("Audio Error"), _("Please fix the following error and restart OpenShot\n%s") % error)
+        QMessageBox.warning(self.parent, _("Audio Error"), _("Please fix the following error and restart OpenShot\n%s") % error)
+
+    def Stop(self):
+        """Disconnect preview parent from update manager and stop worker thread"""
+        get_app().updates.disconnect_listener(self)
+
+        # Stop preview thread (and wait for it to end)
+        self.worker.Stop()
+        self.worker.kill()
+        self.background.exit()
+        self.background.wait(5000)
 
     @pyqtSlot(object, object)
-    def Init(self, parent, timeline, video_widget):
+    def Init(self, parent, timeline, video_widget, max_length=1):
         # Important vars
         self.parent = parent
         self.timeline = timeline
+        self.timeline_max_length = max_length
 
         # Background Worker Thread (for preview video process)
         self.background = QThread(self)
@@ -106,6 +139,9 @@ class PreviewParent(QObject):
         self.worker.moveToThread(self.background)
         self.background.start()
 
+        # Add preview parent as listener for updates
+        get_app().updates.add_listener(self)
+
 
 class PlayerWorker(QObject):
     """ QT Player Worker Object (to preview video on a separate thread) """
@@ -130,11 +166,65 @@ class PlayerWorker(QObject):
         self.number = None
         self.current_frame = None
         self.current_mode = None
-        self.timeline_length = -1
 
         # Create QtPlayer class from libopenshot
         self.player = openshot.QtPlayer()
 
+    def CheckAudioDevice(self):
+        """Check if any audio devices initialization errors, default sample rate, and current open audio device"""
+        # Check audio init error
+        audio_error = self.player.GetError()
+        if audio_error:
+            log.warning('Audio initialization error: %s', audio_error)
+            self.error_found.emit(audio_error)
+
+        # Check active sample rate from audio device
+        # Parse string as float ("48000.0" -> 48000   OR   NaN)
+        detected_sample_rate = float(self.player.GetDefaultSampleRate())
+        if detected_sample_rate and not math.isnan(detected_sample_rate) and detected_sample_rate > 0.0:
+            # Convert float to Integer
+            detected_sample_rate_int = round(detected_sample_rate)
+
+            s = get_app().get_settings()
+            settings_sample_rate = int(s.get("default-samplerate") or 48000)
+            if detected_sample_rate_int != settings_sample_rate:
+                log.warning("Your sample rate (%d) does not match OpenShot (%d). "
+                            "Adjusting your 'Preferences->Preview->Default Sample Rate to match your "
+                            "system rate: %d." % (detected_sample_rate_int,
+                                                  settings_sample_rate,
+                                                  detected_sample_rate_int))
+
+                # Update default sample rate in settings
+                s.set("default-samplerate", detected_sample_rate_int)
+
+                # Update current project's sample rate, so we don't have some crazy
+                # audio drift due to mis-matching sample rates
+                get_app().updates.update(["sample_rate"], detected_sample_rate_int)
+
+        # Convert float 'settings' sample rate to Integer, if detected
+        if type(s.get("default-samplerate")) == float:
+            s.set("default-samplerate", detected_sample_rate_int)
+
+        # Convert float 'project' sample rate to Integer, if detected
+        if type(get_app().project.get("sample_rate")) == float:
+            get_app().updates.update(["sample_rate"], round(get_app().project.get("sample_rate")))
+
+        # Check active audio device name and type from audio device
+        active_audio_device = self.player.GetCurrentAudioDevice()
+        audio_device_value = f"{active_audio_device.get_name()}||{active_audio_device.get_type()}"
+        if s.get("playback-audio-device") != audio_device_value:
+            log.warning("Your active audio device (%s) does not match OpenShot (%s). "
+                        "Adjusting your 'Preferences->Playback->Audio Device' to match your "
+                        "active audio device: %s" % (audio_device_value,
+                                                     s.get("playback-audio-device"),
+                                                     audio_device_value))
+            s.set("playback-audio-device", audio_device_value)
+
+            # Set libopenshot settings
+            lib_settings = openshot.Settings.Instance()
+            lib_settings.PLAYBACK_AUDIO_DEVICE_NAME = active_audio_device.get_name()
+            lib_settings.PLAYBACK_AUDIO_DEVICE_TYPE = active_audio_device.get_type()
+
     @pyqtSlot()
     def Start(self):
         """ This method starts the video player """
@@ -149,9 +239,9 @@ class PlayerWorker(QObject):
         self.player.Pause()
 
         # Check for any Player initialization errors (only JUCE errors bubble up here now)
-        if self.player.GetError():
-            # Emit error_found signal
-            self.error_found.emit(self.player.GetError())
+        # But slightly delay, to allow for correct audio thread initialization with the
+        # correct number of channels and sample rate
+        QTimer.singleShot(1000, self.CheckAudioDevice)
 
         # Main loop, waiting for frames to process
         while self.is_running:
@@ -213,7 +303,7 @@ class PlayerWorker(QObject):
         # Mark frame number for processing (if parent is done initializing)
         self.Seek(self.player.Position())
 
-        log.info("player Position(): %s", self.player.Position())
+        log.debug("player Position(): %s", self.player.Position())
 
     def LoadFile(self, path=None):
         """ Load a media file into the video player """
@@ -248,14 +338,16 @@ class PlayerWorker(QObject):
 
             # Get some settings from the project
             fps = project.get("fps")
-            width = project.get("width")
-            height = project.get("height")
-            sample_rate = project.get("sample_rate")
-            channels = project.get("channels")
-            channel_layout = project.get("channel_layout")
+            width = int(project.get("width"))
+            height = int(project.get("height"))
+            sample_rate = int(project.get("sample_rate"))
+            channels = int(project.get("channels"))
+            channel_layout = int(project.get("channel_layout"))
 
             # Create an instance of a libopenshot Timeline object
-            self.clip_reader = openshot.Timeline(width, height, openshot.Fraction(fps["num"], fps["den"]), sample_rate, channels, channel_layout)
+            self.clip_reader = openshot.Timeline(width, height,
+                                                 openshot.Fraction(fps["num"], fps["den"]),
+                                                 sample_rate, channels, channel_layout)
             self.clip_reader.info.channel_layout = channel_layout
             self.clip_reader.info.has_audio = True
             self.clip_reader.info.has_video = True
@@ -294,12 +386,9 @@ class PlayerWorker(QObject):
         # Seek to frame 1, and resume speed
         self.Seek(seek_position)
 
-    def Play(self, timeline_length):
+    def Play(self):
         """ Start playing the video player """
 
-        # Set length of timeline in frames
-        self.timeline_length = timeline_length
-
         # Start playback
         if self.parent.initialized:
             self.player.Play()
