@@ -29,13 +29,15 @@ from PyQt5.QtCore import Qt, QSize, QPoint
 from PyQt5.QtGui import QDrag
 from PyQt5.QtWidgets import QTreeView, QAbstractItemView, QMenu, QSizePolicy
 
+from classes import info
 from classes.app import get_app
 from classes.logger import log
 
 
 class TransitionsTreeView(QTreeView):
     """ A TreeView QWidget used on the main window """
-    drag_item_size = 48
+    drag_item_size = QSize(48, 48)
+    drag_item_center = QPoint(24, 24)
 
     def contextMenuEvent(self, event):
         # Set context menu mode
@@ -67,8 +69,8 @@ class TransitionsTreeView(QTreeView):
         # Start drag operation
         drag = QDrag(self)
         drag.setMimeData(self.model().mimeData(selected))
-        drag.setPixmap(icon.pixmap(QSize(self.drag_item_size, self.drag_item_size)))
-        drag.setHotSpot(QPoint(self.drag_item_size / 2, self.drag_item_size / 2))
+        drag.setPixmap(icon.pixmap(self.drag_item_size))
+        drag.setHotSpot(self.drag_item_center)
         drag.exec_()
 
     def refresh_columns(self):
@@ -77,6 +79,8 @@ class TransitionsTreeView(QTreeView):
             # Only execute when the treeview is active
             self.hideColumn(2)
             self.hideColumn(3)
+            self.setColumnWidth(0, 80)
+        self.sortByColumn(1, Qt.AscendingOrder)
 
     def __init__(self, model):
         # Invoke parent init
@@ -100,9 +104,10 @@ class TransitionsTreeView(QTreeView):
         self.setSelectionMode(QAbstractItemView.SingleSelection)
         self.setSelectionBehavior(QAbstractItemView.SelectRows)
         self.setSelectionModel(self.transition_model.selection_model)
+        self.setSortingEnabled(True)
 
         # Setup header columns
-        self.setIconSize(QSize(75, 62))
+        self.setIconSize(info.TREE_ICON_SIZE)
         self.setIndentation(0)
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
         self.setWordWrap(True)
